package com.mycompany.commerce.subscription.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2010
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.logging.Level;

import com.ibm.commerce.command.TaskCommandImpl;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.foundation.common.datatypes.CommerceFoundationFactory;
import com.ibm.commerce.foundation.common.datatypes.QuantityType;
import com.ibm.commerce.foundation.common.datatypes.ScheduleEndInfoType;
import com.ibm.commerce.foundation.common.datatypes.ScheduleInfoType;
import com.ibm.commerce.foundation.common.util.sdo.SDODataTypeHelper;
import com.ibm.commerce.foundation.internal.common.exception.FoundationApplicationException;
import com.ibm.commerce.foundation.internal.common.exception.FoundationSystemException;
import com.ibm.commerce.foundation.logging.LoggingHelper;
import com.ibm.commerce.subscription.commands.ScheduleEvaluateCmd;
import com.ibm.commerce.subscription.facade.SubscriptionFacadeConstants;
import com.ibm.commerce.subscription.facade.datatypes.SubscriptionFactory;
import com.ibm.commerce.subscription.logging.SubscriptionApplicationMessageKeys;
import com.mycompany.commerce.subscription.facade.SubscriptionFacadeConstantsExt;

/**
 * This class evaluates the schedule information where in it uses the
 * start date and frequency and computes the next occurrence.
 * 
 * If the schedule is not time-bound, this class sets the end date to
 * {@link SubscriptionFacadeConstants#MAXIMUM_DATE}.
 * 
 * If the scheduled has duration, i.e time bound, this class computes 
 * the end date and total occurrences.
 * 
 * When the command is set to compute the next occurrence from the previous
 * occurrence, this class computes the next occurrence based on the frequency. 
 */
public class DayOfMonthSchedulerEvaluateCmdImpl extends TaskCommandImpl implements
			ScheduleEvaluateCmd {

	private static final java.util.logging.Logger LOGGER = LoggingHelper.getLogger(DayOfMonthSchedulerEvaluateCmdImpl.class);
	
	private static final String CLASSNAME = DayOfMonthSchedulerEvaluateCmdImpl.class.getName();

	private static final Integer COMPUTE_NEXT_OCCURENCE = new Integer(0);

	private static final Integer COMPUTE_FULL_SCHEDULE = new Integer(1);

	private ScheduleInfoType iScheduleInfoToEvaluate;

	private Integer iEvaluateMode;

	private Date iPreviousOccurrence;
	
	private int daysInMonth[] = {31,28,31,30,31,30,31,31,30,31,30,31};
	
	/**
	 * Default constructor for the class.
	 */
	public DayOfMonthSchedulerEvaluateCmdImpl(){
		super();
	}

	/**
	 * This method computes the schedule using the schedule information provided
	 * through the <code>ScheduleInfoType</code> passed as parameter.
	 * 
	 * @throws FoundationApplicationException
	 */
	public void computeFullSchedule() throws FoundationApplicationException {
		final String METHODNAME = "computeSchedule";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		Integer dayOfMonth = Integer.parseInt(String.valueOf(iScheduleInfoToEvaluate.getUserData().getUserDataField().get(SubscriptionFacadeConstantsExt.DAY_OF_MONTH)));
		
		Calendar startDate = Calendar.getInstance();
		startDate.setTime(SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getStartInfo().getStartDate()));
		
		Calendar scheduleDate = Calendar.getInstance();
		
		if(dayOfMonth < startDate.get(Calendar.DATE)){
			if(startDate.get(Calendar.MONTH) <= Calendar.DECEMBER){
				scheduleDate.set(Calendar.MONTH,startDate.get(Calendar.MONTH)+1);
				scheduleDate.set(Calendar.YEAR,startDate.get(Calendar.YEAR));
			}
			else{
				scheduleDate.set(Calendar.MONTH,1);
				scheduleDate.set(Calendar.YEAR,startDate.get(Calendar.YEAR)+1);
			}
		}
		else{
			scheduleDate.set(Calendar.MONTH,startDate.get(Calendar.MONTH));
			scheduleDate.set(Calendar.YEAR,startDate.get(Calendar.YEAR));
		}
		
		daysInMonth[1] += new GregorianCalendar().isLeapYear(scheduleDate.get(Calendar.YEAR)) ? 1 : 0;
		
		int month = scheduleDate.get(Calendar.MONTH);
		if(dayOfMonth <= daysInMonth[month]){
			scheduleDate.set(Calendar.DATE,dayOfMonth);
		}
		else{
			scheduleDate.set(Calendar.DATE,daysInMonth[month]);
		}
		
		
		iScheduleInfoToEvaluate.getFrequencyInfo().setNextOccurence(SDODataTypeHelper.formatXMLDateTime(scheduleDate.getTime()));
		
		ScheduleEndInfoType endInfo = iScheduleInfoToEvaluate.getEndInfo();
		Date endDate = null;
		try{
			endDate = new SimpleDateFormat(SubscriptionFacadeConstants.DATE_FORMAT).parse(SubscriptionFacadeConstants.MAXIMUM_DATE);
		}
		catch(ParseException e){
			if (LoggingHelper.isTraceEnabled(LOGGER)) {
				LOGGER.logp(Level.FINE, CLASSNAME, METHODNAME,"An exception occured while parsing the end date", e);
			}
			throw new FoundationApplicationException(e);
		}
		
		if (endInfo != null){
			if (endInfo.getDuration() != null && endInfo.getDuration().isSetValue()) {
				//to calculate end date and total occurrences from duration
				
				QuantityType duration = endInfo.getDuration();
				QuantityType frequency = iScheduleInfoToEvaluate.getFrequencyInfo().getFrequency();
				
				//Date startDate = SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getStartInfo().getStartDate());
				
				//calculate end date.
				endDate = getEndDate(scheduleDate.getTime(), duration.getValue(),duration.getUom());
				endInfo.setEndDate(SDODataTypeHelper.formatXMLDateTime(endDate));
				
				//calculate the number of occurrences.
				//$ANALYSIS-IGNORE
				int numOfOccurrences = getNumberOfOccurrences(scheduleDate.getTime(), endDate,(int)frequency.getValue(), frequency.getUom());
				iScheduleInfoToEvaluate.getFrequencyInfo().setRemainingOccurrences(BigInteger.valueOf(numOfOccurrences));
			} 
			else {
				if (LoggingHelper.isTraceEnabled(LOGGER)) {
					LOGGER.logp(Level.FINE,CLASSNAME,METHODNAME,"Infinitely recurring schedule");
				}
				endInfo.setEndDate(SDODataTypeHelper.formatXMLDateTime(endDate));
			}
		} 
		else {
			if (LoggingHelper.isTraceEnabled(LOGGER)) {
				LOGGER.logp(Level.FINE,CLASSNAME,METHODNAME,"Infinitely recurring schedule");
			}
			endInfo = getCommerceFoundationFactory().createScheduleEndInfoType();
			endInfo.setEndDate(SDODataTypeHelper.formatXMLDateTime(endDate));
			iScheduleInfoToEvaluate.setEndInfo(endInfo);
		}
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
		
	}
	
	/**
	 * This method computes the number of occurrences using the frequency
	 * specified.
	 * 
	 * @param startDate The start date.
	 * @param endDate The end date.
	 * @param frequencyUOM The unit of measure for the frequency.
	 * @param frequency The value of the frequency.
	 * @return The number of occurrences.
	 */
	private int getNumberOfOccurrences(Date startDate, Date endDate, int frequency, String frequencyUOM) {
		final String METHODNAME = "getNumberOfOccurrences";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		int numberOfOccurrences = 0;
		
		Calendar calStartDate = Calendar.getInstance();
		calStartDate.setTime(startDate);
		
		Calendar calEndDate = Calendar.getInstance();
		calEndDate.setTime(endDate);
		
		Calendar date = (Calendar) calStartDate.clone();
		int calendarField = getUnitOfMeasure(frequencyUOM).getCalendarField();
		
		while (date.before(calEndDate)) {
			date.add(calendarField,frequency);
			numberOfOccurrences++;
		}
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME, numberOfOccurrences);
		}
		return numberOfOccurrences;
	}
	
	/**
	 * This method calculates the end date from the duration and frequency.
	 * @param startDate 
	 * @param duration
	 * @param durationUOM
	 * @return
	 */
	private Date getEndDate(Date startDate, double duration, String durationUOM) {
		final String METHODNAME = "getEndDate";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		Date endDate = null;
		int calendarField = getUnitOfMeasure(durationUOM).getCalendarField();
		
		Calendar startCal = Calendar.getInstance();
		startCal.setTime(startDate);
		
		Calendar clone = (Calendar) startCal.clone();
		//$ANALYSIS-IGNORE
		clone.add(calendarField, (int)duration);
		
		endDate = clone.getTime();
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME, endDate);
		}
		return endDate;
	}

	/**
	 * This method computes the next occurrence of the schedule from the
	 * previous occurrence and the schedule information specified.
	 * 
	 * @throws FoundationApplicationException
	 */
	public void computeNextOccurrence() throws FoundationApplicationException {
		final String METHODNAME = "computeNextOccurrence()";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		ScheduleInfoType scheduleInfo = iScheduleInfoToEvaluate;
		
		QuantityType quantity = scheduleInfo.getFrequencyInfo().getFrequency();
		//$ANALYSIS-IGNORE
		int frequency = (int) quantity.getValue();
		String frequencyUOM = quantity.getUom();
		Date endDate = SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getEndInfo().getEndDate());
		
		Calendar previousOccurrence = Calendar.getInstance();
		previousOccurrence.setTime(iPreviousOccurrence);
		int calendarField = getUnitOfMeasure(frequencyUOM).getCalendarField();
		
		previousOccurrence.add(calendarField, frequency);
		
		daysInMonth[1] += new GregorianCalendar().isLeapYear(previousOccurrence.get(Calendar.YEAR)) ? 1 : 0;
		
		Integer dayOfMonth = Integer.parseInt(String.valueOf(iScheduleInfoToEvaluate.getUserData().getUserDataField().get(SubscriptionFacadeConstantsExt.DAY_OF_MONTH)).trim());
		int month = previousOccurrence.get(Calendar.MONTH);
		
		if(dayOfMonth <= daysInMonth[month]){
			previousOccurrence.set(Calendar.DATE,dayOfMonth);
		}
		else{
			previousOccurrence.set(Calendar.DATE,daysInMonth[month]);
		}
		
		Date nextOccurrence = previousOccurrence.getTime();
		if (nextOccurrence.before(endDate)) {
			scheduleInfo.getFrequencyInfo().setNextOccurence(SDODataTypeHelper.formatXMLDateTime(previousOccurrence.getTime()));
		}
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}

	/**
	 * This method indicates if the schedule has ended. It should be called only
	 * after the command has been executed successfully. This method expects the
	 * end date to be passed if the evaluation mode is to compute the next
	 * occurrence.
	 * 
	 * @return If the schedule has ended or not.
	 */
	public boolean isScheduleEnded() {
		final String METHODNAME = "isScheduleEnded()";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		boolean isEnded = true;
		Date endDate = SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getEndInfo().getEndDate());
		Date nextOccurrence = SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getFrequencyInfo().getNextOccurence());
		if(endDate != null && nextOccurrence != null){
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(endDate);
			isEnded = calendar.before(nextOccurrence);
		}
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME, isEnded);
		}
		return isEnded;
	}

	/**
	 * This method returns the next occurrence of the schedule from the current
	 * date.
	 * 
	 * @return The next occurrence of the schedule. 
	 */
	public Date nextOccurrence() {
		return SDODataTypeHelper.getXMLDateTime(iScheduleInfoToEvaluate.getFrequencyInfo().getNextOccurence());
	}

	/**
	 * This method sets the evaluation mode of the command to compute the full
	 * schedule.
	 */
	public void setComputeFullSchedule() {
		setEvaluateMode(COMPUTE_FULL_SCHEDULE);
	}

	/**
	 * This method sets the evaluation mode of the command to compute the next
	 * occurrence of the schedule.
	 */
	public void setComputeNextOccurenceFromSchedule() {
		setEvaluateMode(COMPUTE_NEXT_OCCURENCE);
	}

	/**
	 * This method sets the previous occurrence of the schedule for the command
	 * to calculate its next occurrence.
	 * 
	 * @param previousOccurrence
	 *            The previous occurrence of the schedule.
	 */
	public void setPreviousOccurrence(Date previousOccurrence) {
		iPreviousOccurrence = previousOccurrence;
	}

	/**
	 * Sets the schedule to be evaluated.
	 * 
	 * @param scheduleInfo
	 *            The schedule information object to be evaluated.
	 */
	public void setScheduleToEvaluate(ScheduleInfoType scheduleInfo) {
		iScheduleInfoToEvaluate = scheduleInfo;		
	}

	 /**
	 * This method validates if the schedule information provided.
	 * @throws FoundationApplicationException 
	 */
	public void validateSchedule() throws FoundationApplicationException {
		
	}
	
	/**
	 * This method returns the <code>UnitOfMeasure</code> instance for the unit
	 * specified.
	 * 
	 * @param unit The quantity unit ID of the unit of measure.
	 * @return The unit of measure instance.
	 */
	protected static UnitOfMeasure getUnitOfMeasure(String unit) {
		final String METHODNAME = "getUnitOfMeasure";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME, unit);
		}
		UnitOfMeasure[] units = UnitOfMeasure.values();
		UnitOfMeasure unitOfMeasure = null;
		for (UnitOfMeasure uom : units) {
			if (unit.equals(uom.getQuantityUnitId())) {
				unitOfMeasure = uom;
				break;
			}
		}
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME, unitOfMeasure);
		}
		return unitOfMeasure;
	}
	
	/**
	 * Returns an instance of the commerce foundation factory.
	 * 
	 * @return An instance of the commerce foundation factory.
	 */
	protected CommerceFoundationFactory getCommerceFoundationFactory() {
		final String METHODNAME = "getCommerceFoundationFactory()";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
		return CommerceFoundationFactory.eINSTANCE;
	}
	
	/**
	 * Returns an instance of the subscription factory.
	 * 
	 * @return An instance of the subscription factory.
	 */
	protected SubscriptionFactory getSubscriptionFactory() {
		final String METHODNAME = "getSubscriptionFactory()";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
		return SubscriptionFactory.eINSTANCE;
	}
	
	/**
	 * This method checks if the schedule information is provided and calls the
	 * validate schedule method.
	 * 
	 * @throws ECException
	 */
	public void validateParameters() throws ECException {
		final String METHODNAME = "validateParameters";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		if (iScheduleInfoToEvaluate == null) {
			throw new FoundationSystemException(
					SubscriptionApplicationMessageKeys._ERR_EXCEPTION_MISSING_SCHEDULE_INFO_TO_EVALUATE,
					null, CLASSNAME, METHODNAME);
		}
		
		if(iEvaluateMode == null){
			throw new FoundationSystemException(
					SubscriptionApplicationMessageKeys._ERR_EXCEPTION_MISSING_EVALUATE_MODE_FOR_COMMAND,
					new Object[]{iEvaluateMode, CLASSNAME}, CLASSNAME, METHODNAME);
		}
		
		try {
			validateSchedule();
		} catch (FoundationApplicationException e) {
			throw new ECApplicationException(e);
		}
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}
	
	/**
	 * This method invokes the <code>computeSchedule</code> method to compute
	 * the schedule.
	 * 
	 * @throws ECException
	 */
	public void performExecute() throws ECException {
		final String METHODNAME = "performExecute";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		if(getEvaluateMode() == COMPUTE_FULL_SCHEDULE){
			try {
				computeFullSchedule();
			} catch (FoundationApplicationException e) {
				throw new ECApplicationException(e);
			}
		} else if (getEvaluateMode() == COMPUTE_NEXT_OCCURENCE){
			try{
				computeNextOccurrence();
			} catch (FoundationApplicationException e){
				throw new ECApplicationException(e);
			}
		}
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}
	
	/**
	 * This method sets the evaluation mode of the command.
	 * 
	 * @param evaluateMode The evaluation mode of the command.
	 */
	private void setEvaluateMode(Integer evaluateMode) {
		iEvaluateMode = evaluateMode;
	}
	
	/**
	 * Returns the evaluation mode for the command.
	 * @return The evaluation mode for the command.
	 */
	private Integer getEvaluateMode(){
		return iEvaluateMode;
	}	
}