package com.mycompany.commerce.subscription.facade.server.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2010
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.logging.Logger;

import com.ibm.commerce.foundation.common.datatypes.ScheduleInfoType;
import com.ibm.commerce.foundation.common.exception.ApplicationError;
import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.subscription.commands.ScheduleEvaluateCmd.UnitOfMeasure;
import com.ibm.commerce.subscription.facade.datatypes.SubscriptionType;
import com.ibm.commerce.subscription.facade.server.commands.ProcessSubscriptionCreateActionCmdImpl;
import com.ibm.commerce.subscription.facade.server.exception.SubscriptionApplicationException;
import com.ibm.icu.util.Calendar;
import com.mycompany.commerce.subscription.facade.SubscriptionFacadeConstantsExt;
import com.mycompany.commerce.subscription.facade.server.helpers.SubscriptionServerConstantsExt;
import com.mycompany.commerce.subscription.logging.SubscriptionApplicationMessageKeysExt;

/**
 * This class extends the ProcessSubscriptionCreateActionCmdImpl.
 * It checks for the schedule type passed and invokes the corresponding 
 * schedule evaluation command.
 *
 */
public class ProcessSubscriptionCreateActionCmdExtImpl extends
		ProcessSubscriptionCreateActionCmdImpl {
	
	
	private static final String CLASSNAME = ProcessSubscriptionCreateActionCmdExtImpl.class.getName();
	
	private static final Logger LOGGER = LoggingHelper.getLogger(ProcessSubscriptionCreateActionCmdExtImpl.class);
	
	/**
	 * This method validates scheduleType and dayOfMonth parameters passed in 
	 * the fulfillment schedule. 
	 * @param subscription
	 *            The subscription noun being validated.
	 * @param fulfillmentSchedule
	 *            The fulfillment schedule noun part of the subscription noun.
	 * @param isRecurring
	 *            Boolean to indicate if the subscription is a recurring order.
	 * @return A list of application errors, if any. An empty list is returned
	 *         if there are no errors. Possible errors include missing dayOfMonth
	 *         and invalid value of dayOfMonth.
	 * @throws SubscriptionApplicationException
	 * @see {@link ProcessSubscriptionCreateActionCmdImpl#validateFulfillmentSchedule(SubscriptionType,ScheduleInfoType,boolean)}
	 */
	protected Collection<? extends ApplicationError> validateFulfillmentSchedule(
			SubscriptionType subscription,
			ScheduleInfoType fulfillmentSchedule, boolean isRecurring) throws SubscriptionApplicationException{
		
		final String METHODNAME = "validateFulfillmentSchedule(SubscriptionType, SubscriptionFulfillmentDetailsType)";
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		List<ApplicationError> errors = new ArrayList<ApplicationError>();
		
		String scheduleName = SubscriptionServerConstantsExt.DEFAULT_SCHEDULE_TYPE;
		
		String dayOfMonth = null;
		
		//Check for the schedule type in the user data.
		if(subscription.getSubscriptionInfo().getFulfillmentSchedule().getUserData() != null
				&& subscription.getSubscriptionInfo().getFulfillmentSchedule().getUserData().getUserDataField() != null
					&& subscription.getSubscriptionInfo().getFulfillmentSchedule().getUserData().getUserDataField().get(SubscriptionFacadeConstantsExt.SCHEDULE_TYPE) != null){
						
			dayOfMonth = String.valueOf(subscription.getSubscriptionInfo().getFulfillmentSchedule().getUserData().getUserDataField().get(SubscriptionFacadeConstantsExt.DAY_OF_MONTH));
			
			if(dayOfMonth != null && !dayOfMonth.isEmpty()){
				Calendar cal = Calendar.getInstance();
				int maxDay = cal.getMaximum(Calendar.DAY_OF_MONTH);
				
				/**
				 * Error to indicate an invalid day of the month.
				 * If the input value of day of the month is greater than
				 * the maximum allowable value(i.e 31), the method returns an error.
				 */
				
				if(Integer.parseInt(dayOfMonth) > maxDay){
					ApplicationError error = new ApplicationError(
							ApplicationError.TYPE_INVALID_PARAMETER_VALUE,
							SubscriptionApplicationMessageKeysExt._ERR_INVALID_DAYOFMONTH,
							new Object[] { dayOfMonth }, LOGGER
									.getResourceBundleName());
					errors .add(error);
				}
				else{
					fulfillmentSchedule.getFrequencyInfo().getFrequency()
							.setUom(UnitOfMeasure.Months.getQuantityUnitId());
				}
			}
			else{
				//Error to indicate that the day of the month parameter is missing.
				ApplicationError error = new ApplicationError(
						ApplicationError.TYPE_MISSING_PARAMETER,
						SubscriptionApplicationMessageKeysExt._ERR_MISSING_DAYOFMONTH,
						new Object[] { dayOfMonth }, LOGGER
								.getResourceBundleName());
				errors .add(error);
			}
		}
		
		errors.addAll(super.validateFulfillmentSchedule(subscription, fulfillmentSchedule, isRecurring));
		
		if(LoggingHelper.isEntryExitTraceEnabled(LOGGER)){
			LOGGER.exiting(CLASSNAME, METHODNAME, errors);
		}
				
		return errors;
	}
	
	/**
	 * This method checks for the schedule type value in the user data field
	 * and returns the appropriate schedule to be evaluated.
	 * @return String The schedule name to be evaluated.
	 * @see {@link ProcessSubscriptionCreateActionCmdImpl#getScheduleTypeFromScheduleInfo(ScheduleInfoType)}
	 */
	
	protected String getScheduleTypeFromScheduleInfo(ScheduleInfoType schedule) {
		
		final String METHODNAME = "getScheduleTypeFromScheduleInfo(ScheduleInfoType)";
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		String scheduleName = super.getScheduleTypeFromScheduleInfo(schedule);
		
		if (schedule.getUserData() != null
				&& schedule.getUserData().getUserDataField() != null
				&& schedule.getUserData().getUserDataField().get(
						SubscriptionFacadeConstantsExt.SCHEDULE_TYPE) != null) {
				
			scheduleName = SubscriptionServerConstantsExt.DAY_OF_MONTH_SCHEDULE_TYPE;

		}

		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, scheduleName);
		}
		
		return scheduleName;
	}
}
